#define isl_aff_get_constant	isl_gmp_aff_get_constant
#define isl_aff_get_coefficient	isl_gmp_aff_get_coefficient
#define isl_aff_get_denominator	isl_gmp_aff_get_denominator
#define isl_aff_set_constant	isl_gmp_aff_set_constant
#define isl_aff_set_coefficient	isl_gmp_aff_set_coefficient
#define isl_aff_set_denominator	isl_gmp_aff_set_denominator
#define isl_aff_add_constant	isl_gmp_aff_add_constant
#define isl_aff_add_constant_num	isl_gmp_aff_add_constant_num
#define isl_aff_add_coefficient	isl_gmp_aff_add_coefficient
#define isl_aff_mod	isl_gmp_aff_mod
#define isl_aff_scale	isl_gmp_aff_scale
#define isl_aff_scale_down	isl_gmp_aff_scale_down
#define isl_pw_aff_mod	isl_gmp_pw_aff_mod
#define isl_pw_aff_scale	isl_gmp_pw_aff_scale
#define isl_pw_aff_scale_down	isl_gmp_pw_aff_scale_down
#define isl_multi_aff_scale	isl_gmp_multi_aff_scale
#define isl_ast_expr_get_int	isl_gmp_ast_expr_get_int
#define isl_constraint_get_constant	isl_gmp_constraint_get_constant
#define isl_constraint_get_coefficient	isl_gmp_constraint_get_coefficient
#define isl_constraint_set_constant	isl_gmp_constraint_set_constant
#define isl_constraint_set_coefficient	isl_gmp_constraint_set_coefficient
#define isl_basic_set_max	isl_gmp_basic_set_max
#define isl_set_min	isl_gmp_set_min
#define isl_set_max	isl_gmp_set_max
#define isl_gmp_hash	isl_gmp_gmp_hash
#define isl_basic_map_plain_is_fixed	isl_gmp_basic_map_plain_is_fixed
#define isl_map_fix	isl_gmp_map_fix
#define isl_map_plain_is_fixed	isl_gmp_map_plain_is_fixed
#define isl_map_fixed_power	isl_gmp_map_fixed_power
#define isl_mat_get_element	isl_gmp_mat_get_element
#define isl_mat_set_element	isl_gmp_mat_set_element
#define isl_point_get_coordinate	isl_gmp_point_get_coordinate
#define isl_point_set_coordinate	isl_gmp_point_set_coordinate
#define isl_qpolynomial_rat_cst_on_domain	isl_gmp_qpolynomial_rat_cst_on_domain
#define isl_qpolynomial_is_cst	isl_gmp_qpolynomial_is_cst
#define isl_qpolynomial_scale	isl_gmp_qpolynomial_scale
#define isl_term_get_num	isl_gmp_term_get_num
#define isl_term_get_den	isl_gmp_term_get_den
#define isl_qpolynomial_fold_scale	isl_gmp_qpolynomial_fold_scale
#define isl_pw_qpolynomial_fold_fix_dim	isl_gmp_pw_qpolynomial_fold_fix_dim
#define isl_basic_set_fix	isl_gmp_basic_set_fix
#define isl_set_lower_bound	isl_gmp_set_lower_bound
#define isl_set_upper_bound	isl_gmp_set_upper_bound
#define isl_set_fix	isl_gmp_set_fix
#define isl_set_plain_is_fixed	isl_gmp_set_plain_is_fixed
#define isl_union_map_fixed_power	isl_gmp_union_map_fixed_power
#define isl_val_int_from_isl_int	isl_gmp_val_int_from_isl_int
#define isl_val_get_num_isl_int	isl_gmp_val_get_num_isl_int
#define isl_vec_get_element	isl_gmp_vec_get_element
#define isl_vec_set_element	isl_gmp_vec_set_element
#define isl_vec_set	isl_gmp_vec_set
#define isl_vec_fdiv_r	isl_gmp_vec_fdiv_r
