<?php

/**
 * This file is automatically generated. Use 'arc liberate' to rebuild it.
 * @generated
 * @phutil-library-version 2
 */

phutil_register_library_map(array(
  '__library_version__' => 2,
  'class' =>
  array(
    'LitTestEngine' => 'src/LitTestEngine.php',
  ),
  'function' =>
  array(
  ),
  'xmap' =>
  array(
    'LitTestEngine' => 'ArcanistBaseUnitTestEngine',
  ),
));
