//===--------------- polly/Options.h - The Polly option category *- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// Introduce an option category for Polly.
//
//===----------------------------------------------------------------------===//

#ifndef POLLY_OPTIONS_H
#define POLLY_OPTIONS_H

#include "llvm/Support/CommandLine.h"

extern llvm::cl::OptionCategory PollyCategory;
#endif
